"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDefaultGuard = exports.combineGuards = exports.createMixedGuard = exports.createArrayGuard = exports.createObjectGuard = exports.createEnumGuard = exports.FileGuard = exports.NumberGuard = exports.BooleanGuard = exports.StringGuard = void 0;
const a = typeof ("Asd");
function createPlainGuard(type) {
    return (value) => {
        if (typeof value === type) {
            return value;
        }
        else {
            return new Error(`${value} is not assignable to type ${type}`);
        }
    };
}
exports.StringGuard = createPlainGuard("string");
exports.BooleanGuard = createPlainGuard("boolean");
const NumberGuard = (value) => {
    const tempo = Number(value);
    if (Object.is(tempo, NaN))
        return new Error(`'${value}' is not assignable to type number`);
    if (tempo < 0)
        return 0;
    return tempo;
};
exports.NumberGuard = NumberGuard;
const FileGuard = (value) => {
    if (value instanceof File) {
        return value;
    }
    else {
        return new Error(`'${value}' is not a File instance`);
    }
};
exports.FileGuard = FileGuard;
const createEnumGuard = (variants) => (value) => {
    if (variants.includes(value))
        return value;
    else
        return new Error(`'${JSON.stringify(variants)}' doesn't includes ${value}`);
};
exports.createEnumGuard = createEnumGuard;
const createObjectGuard = (shape) => {
    const guards = {};
    for (let key in shape) {
        guards[key] = exports.getDefaultGuard(shape[key]);
    }
    return (value) => {
        if (!value)
            return value;
        if (typeof value !== "object")
            return new Error(`${value} is not an object`);
        for (let key in shape) {
            if (!(key in value))
                return new Error(`Prop ${key} is defined in schema, but it is not in value(${value})`);
            const guard = guards[key];
            if (guard) {
                const result = guard(value[key]);
                if (result instanceof Error)
                    return result;
            }
        }
        return value;
    };
};
exports.createObjectGuard = createObjectGuard;
const createArrayGuard = (childrenType) => {
    const guard = exports.getDefaultGuard(childrenType);
    return (value) => {
        if (!value)
            return value;
        if (!(value instanceof Array))
            return new Error(`${value} is not instance of Array`);
        if (!guard)
            return value;
        for (let val of value) {
            const result = guard(val);
            if (result instanceof Error) {
                return result;
            }
        }
        return value;
    };
};
exports.createArrayGuard = createArrayGuard;
const createMixedGuard = (mix) => {
    const guards = [];
    let names = [];
    for (let type of mix) {
        names.push(type.type);
        guards.push(exports.getDefaultGuard(type));
    }
    let types = `[${names.join(", ")}]`;
    return (value) => {
        if (!value)
            return value;
        let val = value;
        for (let guard of guards) {
            if (!guard)
                continue;
            const result = guard(val);
            if (!(result instanceof Error)) {
                return result;
            }
        }
        return new Error(`"${value}" is not assignable to types ${types}`);
    };
};
exports.createMixedGuard = createMixedGuard;
const combineGuards = (...args) => (value) => {
    let val = value;
    for (let validator of args) {
        if (!validator)
            continue;
        const result = validator(val);
        if (result instanceof Error) {
            return result;
        }
        else {
            val = result;
        }
    }
    return val;
};
exports.combineGuards = combineGuards;
const getDefaultGuard = (type) => {
    switch (type.type) {
        case "string": return exports.StringGuard;
        case "number": return exports.NumberGuard;
        case "boolean": return exports.BooleanGuard;
        case "enum": return exports.createEnumGuard(type.variants);
        case "file": return exports.FileGuard;
        case "object": return exports.createObjectGuard(type.fields);
        case "array": return exports.createArrayGuard(type.consistsOf);
        case "mixed": return exports.createMixedGuard(type.mix);
    }
};
exports.getDefaultGuard = getDefaultGuard;
//# sourceMappingURL=Guards.js.map